//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMYDIALOG_H
#define ARMYDIALOG_H

#include <pgwindow.h>
#include <pgeventobject.h>
#include <pgbutton.h>
#include <pgcheckbutton.h>
#include <pglabel.h>
#include "army.h"

/** brief Dialog for level advancement
  * 
  * This dialog is displayed whenever a unit gains enough experience points
  * to advance a level. The user can choose which stat he wants to raise.
  */

class ArmyLevelDialog : public PG_Window, public PG_EventObject
{
    public:
        /** \brief The constructor which sets up the whole thing
          * 
          * @param army     the army which advances
          * @param parent   the parent widget (derived from paragui)
          * @param rect     the rect of this widget (derived from paragui)
          */
        ArmyLevelDialog(Army* army, PG_Widget* parent, PG_Rect rect);
        
        ~ArmyLevelDialog();

        //! Initialises all the labels etc.
        void initStats();
        
        //CALLBACKS and event functions
        //! This function is called when the user selects a stat.
        PARAGUI_CALLBACK(stat_chosen);

        //! This function is called when the user pushes the OK button.
        PARAGUI_CALLBACK(b_okClicked);

    private:
        bool eventKeyDown(const SDL_KeyboardEvent* key);

        Army* d_army;
        PG_Button* d_b_ok;
        PG_Button* d_b_pic;
        PG_Label* d_l_info;
        PG_Label* d_l_name;
        PG_Label* d_l_stats;
        PG_Label* d_l_basic;
        PG_Label* d_l_real;
        PG_Label *d_l_strengthn, *d_l_strengthb, *d_l_strengthr;
        PG_Label* d_l_defensen, *d_l_defenseb, *d_l_defenser;
        PG_Label* d_l_vitalityn, *d_l_vitalityb, *d_l_vitalityr;
        PG_Label* d_l_hpn, *d_l_hpb, *d_l_hpr;
        PG_Label* d_l_movesn, *d_l_movesb, *d_l_movesr;
        PG_Label* d_l_xp;
        PG_Label* d_l_level1;
        PG_Label* d_l_level2;

        PG_RadioButton* d_choose_def;
        PG_RadioButton* d_choose_vit;
        PG_RadioButton* d_choose_hp;
        PG_RadioButton* d_choose_str;
        PG_RadioButton* d_choose_mov;
};

#endif
