//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <pgapplication.h>
#include <pgmessagebox.h>
#include "RMessage.h"
#include "defs.h"
#include "playerlist.h"

RMessage::RMessage(std::string message)
    :Reaction(MESSAGE), d_message(message)
{
}

RMessage::RMessage(XML_Helper* helper)
    :Reaction(MESSAGE)
{
    helper->getData(d_message, "message");
}

RMessage::~RMessage()
{
}

bool RMessage::save(XML_Helper* helper)
{
    bool retval = true;

    retval &= helper->openTag("reaction");
    retval &= helper->saveData("type", std::string("RMESSAGE"));
    retval &= helper->saveData("message", d_message);
    retval &= helper->closeTag();
        
    return retval;
}

bool RMessage::trigger()
{
    //we don't show messages for computer players
    Player::TYPE type = Playerlist::getActiveplayer()->getType();
    if (type == Player::AI_FAST || type == Player::AI_SMART || type == Player::AI_DUMMY || type == Player::NEUTRAL)
        return true;
    
    //Display a message box for the beginning in the middle of the screen
    int height = 200;
    int width = 400;
    int x = (PG_Application::GetScreenWidth() - width)/2;
    int y = (PG_Application::GetScreenHeight() - height)/2;

    PG_MessageBox mb(0, PG_Rect(x,y,width,height), 0, d_message.c_str(),
                     PG_Rect(150, 170, 100, 20), _("OK"));
    mb.Show();
    mb.WaitForClick();
    mb.Hide();
    
    return true;
}
