//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <sstream>
#include "army.h"
#include "armyset.h"
#include "armysetlist.h"
#include "counter.h"
#include "GraphicsCache.h"
#include "player.h"

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
#define debug(x)

Army::Army(Armyset* armyset, int type, Player* owner)
    :Armyset_army(armyset->army[type]), d_armyset(armyset), d_owner(owner),
    d_type(type), d_hp(getOrigMaxHp()), d_moves(getOrigMaxMoves()),
     d_xp(0), d_level(1), d_blessed(false), d_grouped(true),
     d_battles_number(0), d_number_hashit(0),
     d_number_hasbeenhit(0),d_number_attacks(0) 
{
    d_id = fl_counter->getNextId();

    for(int i=0;i<3;i++)
    {
        d_medal_bonus[i]=false;
        debug("ARMYCONSTRUCTOR medalsbonus[" << i << "]=" << d_medal_bonus[i])
    }
}

Army::Army(Army& a)
    :Armyset_army(a), d_armyset(a.d_armyset), d_owner(a.d_owner), d_id(a.d_id),
     d_type(a.d_type), d_hp(a.d_hp), d_moves(a.d_moves), 
     d_xp(a.d_xp), d_level(a.d_level), d_blessed(a.d_blessed), d_grouped(a.d_grouped),
     d_battles_number(a.d_battles_number), 
     d_number_hashit(a.d_number_hashit), d_number_hasbeenhit(a.d_number_hasbeenhit),
     d_number_attacks(a.d_number_attacks)
{
    for(int i=0;i<3;i++)
    {
        d_medal_bonus[i]=a.d_medal_bonus[i];
        debug("ARMY-COPY-CONSTRUCTOR medalsbonus[" << i << "]=" << d_medal_bonus[i])
    }  
}

Army::Army(XML_Helper* helper)
  :Armyset_army(), d_grouped(true),
   d_number_hashit(0),
   d_number_hasbeenhit(0),d_number_attacks(0)
{
    std::string armyset;
    
    //get the information which army we are
    helper->getData(d_type, "type");
    helper->getData(armyset, "armyset");

    //now load the correct army information
    d_armyset = Armysetlist::getInstance()->getArmyset(armyset);
    copyVals(&d_armyset->army[d_type]);

    //adjust the values with the saved data
    helper->getData(d_id, "id");
    helper->getData(d_hp, "hp");
    helper->getData(d_strength, "strength");
    helper->getData(d_defense, "defense");
    helper->getData(d_vitality, "vitality");
    helper->getData(d_max_hp, "maxhp");
    helper->getData(d_moves, "moves");
    helper->getData(d_xp, "xp");
    helper->getData(d_xp_value,"expvalue");
    helper->getData(d_level, "level");
    helper->getData(d_blessed, "blessed");

    std::string medals;
    std::stringstream smedals;
    bool val;
    
    helper->getData(medals, "medals");    
    smedals.str(medals);

    for(int i=0;i<3;i++)
    {
        smedals >> val;
        d_medal_bonus[i]=val;
        debug("ARMY-XML-CONSTRUCTOR medalsbonus[" << i << "]=" << d_medal_bonus[i])
    }

    helper->getData(d_battles_number, "battlesnumber");    
}

Army::~Army()
{
    sdying.emit(this);
}

SDL_Surface* Army::getPixmap()
{
    //use the GraphicsCache to get a picture of the army's armyset_army
    return GraphicsCache::getInstance()->getArmyPic(d_armyset->getName(),
                                                    d_type, d_owner,getLevel(),
                                                    getMedalBonuses());
}

void Army::resetMoves()
{
    d_moves = getMaxMoves();
}

void Army::bless()
{
    if (!d_blessed)
    {
        d_blessed = true;
        d_strength++;
    }
}

void Army::heal(int hp)
{
    if (hp > 0)
    {
    //add no_hp
    d_hp += hp;
    }
    else
    {
    //healing at end of turn, army has to take care of how much to heal
    //for the beginning, we heal 1/5th of hp's every turn
    // (should be changed somewhen)
    d_hp += (getMaxHp() / 5);
    }

    if (d_hp > getMaxHp())
    d_hp = getMaxHp();
}

bool Army::damage(int damageDone)
{
    d_hp -= damageDone;
    return (d_hp <= 0);
}

void Army::decrementMoves(int moves)
{
    d_moves -= moves;

    //just to make sure
    if (d_moves < 0)
    {
        std::cerr <<_("Error: moves of army < 0\n");
        d_moves = 0;
    }
}


void Army::gainXp(double n)
{
    d_xp += n;
}

bool Army::canGainLevel()
{
    int tmp=(int) getXP()/getLevel();

    if (((int) tmp/10)>=1)
        return true;
    else
        return false;
}

void Army::gainLevel()
{
    d_level++;
}

bool Army::save(XML_Helper* helper)
{
    bool retval = true;

    retval &= helper->openTag("army");

    retval &= saveData(helper);
    
    retval &= helper->closeTag();

    return retval;
}

bool Army::saveData(XML_Helper* helper)
{
    bool retval = true;
    
    retval &= helper->saveData("id", d_id);
    retval &= helper->saveData("armyset", d_armyset->getName());
    retval &= helper->saveData("type", d_type);
    retval &= helper->saveData("hp", d_hp);
    retval &= helper->saveData("strength", d_strength);
    retval &= helper->saveData("defense", d_defense);
    retval &= helper->saveData("vitality", d_vitality);
    retval &= helper->saveData("maxhp", d_max_hp);
    retval &= helper->saveData("moves", d_moves);
    retval &= helper->saveData("xp", d_xp);
    retval &= helper->saveData("expvalue", getXpReward());
    retval &= helper->saveData("level", d_level);
    retval &= helper->saveData("blessed", d_blessed);

    std::stringstream medals;
    for (int i=0;i<3;i++)
    {
        medals << d_medal_bonus[i] << " ";
    }
    retval &= helper->saveData("medals", medals.str());
    retval &= helper->saveData("battlesnumber",d_battles_number);    

    return retval;
}

int Army::getStrength()
{
    return d_strength;
}

int Army::getDefense()
{
    return d_defense;
}

int Army::getVitality()
{
    return d_vitality;
}

int Army::getMaxHp()
{
    return d_max_hp;
}


int Army::getMaxMoves()
{
    return d_max_moves;
}

unsigned int Army::getMoveBonus()
{
    return d_move_bonus;
}

unsigned int Army::getArmyBonus()
{
    return d_army_bonus;
}

void  Army::printAllDebugInfo()
{
   printDebugInfo();

   std::cerr << "type = "    << d_type    << std::endl;
   std::cerr << "level = "   << d_level   << std::endl;
   std::cerr << "xp = "      << d_xp      << std::endl;
   std::cerr << "blessed = " << d_blessed << std::endl;
   std::cerr << "grouped = " << d_grouped << std::endl;

   std::cerr << "medal[0] = " << d_medal_bonus[0] << std::endl;
   std::cerr << "medal[1] = " << d_medal_bonus[1] << std::endl;
   std::cerr << "medal[2] = " << d_medal_bonus[2] << std::endl;

   std::cerr << "battle number = "     << d_battles_number    << std::endl;
   std::cerr << "has hit = "           << d_number_hashit     << std::endl;
   std::cerr << "has been hit = "      << d_number_hasbeenhit << std::endl;
   std::cerr << "number of attacks = " << d_number_attacks    << std::endl;

}
