//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "stack.h"
#include <sstream>
#include <assert.h>
#include "playerlist.h"
#include "path.h"
#include "armysetlist.h"
#include "counter.h"
#include "army.h"
#include "hero.h"
#include "GameMap.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
#define debug(x)

Stack::Stack(Player* player, PG_Point pos)
    :d_player(player), d_defending(false), d_pos(pos), d_deleting(false)
{
    d_path = new Path();
    d_id = fl_counter->getNextId();
}

Stack::Stack(Stack& s)
    :d_id(s.d_id), d_player(s.d_player), d_defending(s.d_defending), d_pos(s.d_pos),
     d_deleting(false)
{
    clear();
    d_path = new Path();
    //deep copy the other stack's armies
    for (iterator sit = s.begin(); sit != s.end(); sit++)
    {
        Army* a;
        a = new Army((**sit));
        push_back(a);
    }
}

Stack::Stack(XML_Helper* helper)
    :d_deleting(false)
{
    int i;

    helper->getData(d_id, "id");
    helper->getData(i, "x");
    d_pos.x = i;
    helper->getData(i, "y");
    d_pos.y = i;
    helper->getData(d_defending, "defending");

    helper->getData(i, "player");
    d_player = Playerlist::getInstance()->getPlayer(static_cast<unsigned int>(i));


    helper->registerTag("path", SigC::slot((*this), &Stack::load));
    helper->registerTag("army", SigC::slot((*this), &Stack::load));
    helper->registerTag("hero", SigC::slot((*this), &Stack::load));

}

Stack::~Stack()
{
    d_deleting = true;
    delete d_path;
    flClear();
}

// set the stack to the next position of the path

bool Stack::moveOneStep()
{
    debug("move_one_step()");

    d_pos = **d_path->begin();

    //now remove first point of the path
    d_path->fl_erase(d_path->begin());

    return true;
}

// return the maximum moves of this stack by checking the moves of each army

int Stack::getGroupMoves()
{
    int maximum = 100;

    for (iterator it = begin(); it != end(); it++)
    {
        if ((*it)->getMoves() < maximum)
        {
            maximum = (*it)->getMoves();
        }
    }

    return maximum;
}

int Stack::getMinTilesAroundMoves(int x, int y)
{

  GameMap *tmp=GameMap::getInstance();
  int tmpmove=32767;
  int mapsize=tmp->getSize();
  Path * tmppath =new Path();
  PG_Point* p = new PG_Point;

  tmppath->checkPath(this);

  if (x>0 && x<mapsize && y>0 && y<mapsize)
  {
    debug("x>0<mapsize y>0<mapsize  TMPMOVE= " << tmpmove)

    p->x=x+1;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p))
    {
        tmpmove=tmp->getTile(x+1,y+1)->getMoves();
        debug("Checked Position: " << x+1 << " " << y+1 )
    }
    p->x=x;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y+1)->getMoves();
        debug("Checked Position: " << x << " " << y+1 )
    }
    p->x=x+1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y)->getMoves();
        debug("Checked Position: " << x+1 << " " << y )
    }
    p->x=x;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y-1)->getMoves();
        debug("Checked Position: " << x << " " << y-1 )
    }
    p->x=x+1;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y-1)->getMoves();
        debug("Checked Position: " << x+1 << " " << y-1 )
    }
    p->x=x-1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y)->getMoves();
        debug("Checked Position: " << x-1 << " " << y )
    }
    p->x=x-1;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y-1)->getMoves();
        debug("Checked Position: " << x-1 << " " << y-1 )
    }
    p->x=x-1;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y+1)->getMoves();
        debug("Checked Position: " << x-1 << " " << y+1 )
    }
    delete tmppath;
    delete p;
    if (tmpmove!=32767)
        return tmpmove;
    else
        return 0;
  }


  if (x==0 && y==0)
  {
    debug("x=0 y=0 TMPMOVE= " << tmpmove)

    p->x=x+1;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p))
    {
        tmpmove=tmp->getTile(x+1,y+1)->getMoves();
        debug("Checked Position: " << x+1 << " " << y+1 )
    }
    p->x=x;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y+1)->getMoves();
        debug("Checked Position: " << x << " " << y+1 )
    }
    p->x=x+1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y)->getMoves();
        debug("Checked Position: " << x+1 << " " << y )
    }
    delete tmppath;
    delete p;
    if (tmpmove!=32767)
        return tmpmove;
    else
        return 0;
  }


  if (x==mapsize && y==mapsize)
  {
    debug("x=mapsize y=mapsize  TMPMOVE= " << tmpmove)

    p->x=x-1;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p))
    {
        tmpmove=tmp->getTile(x-1,y-1)->getMoves();
        debug("Checked Position: " << x-1 << " " << y-1 )
    }
    p->x=x;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y-1)->getMoves();
        debug("Checked Position: " << x << " " << y-1 )
    }
    p->x=x-1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y)->getMoves();
        debug("Checked Position: " << x-1 << " " << y )
    }
    delete tmppath;
    delete p;
    if (tmpmove!=32767)
        return tmpmove;
    else
        return 0;
  }


  if (x==mapsize && y<mapsize && y>0)
  {
    debug("x=mapsize y>0<mapsize  TMPMOVE= " << tmpmove)

    p->x=x-1;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p))
    {
        tmpmove=tmp->getTile(x-1,y-1)->getMoves();
        debug("Checked Position: " << x-1 << " " << y-1 )
    }
    p->x=x;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y-1)->getMoves();
        debug("Checked Position: " << x << " " << y-1 )
    }
    p->x=x-1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y)->getMoves();
        debug("Checked Position: " << x-1 << " " << y )
    }
    p->x=x-1;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y+1)->getMoves();
        debug("Checked Position: " << x-1 << " " << y+1 )
    }
    p->x=x;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y+1)->getMoves();
        debug("Checked Position: " << x << " " << y+1 )
    }
    delete tmppath;
    delete p;
    if (tmpmove!=32767)
        return tmpmove;
    else
        return 0;
  }


  if (x<mapsize && x>0 && y==mapsize)
  {
    debug("x>0<mapsize y=mapsize  TMPMOVE= " << tmpmove)

    p->x=x-1;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p))
    {
        tmpmove=tmp->getTile(x-1,y-1)->getMoves();
        debug("Checked Position: " << x-1 << " " << y-1 )
    }
    p->x=x;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y-1)->getMoves();
        debug("Checked Position: " << x << " " << y-1 )
    }
    p->x=x-1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y)->getMoves();
        debug("Checked Position: " << x-1 << " " << y )
    }
    p->x=x+1;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y-1)->getMoves();
        debug("Checked Position: " << x+1 << " " << y-1 )
    }

    p->x=x+1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y)->getMoves();
        debug("Checked Position: " << x+1 << " " << y )
    }
    delete tmppath;
    delete p;
    if (tmpmove!=32767)
        return tmpmove;
    else
        return 0;
  }


  if (x>0 && x<mapsize && y==0)
  {
    debug("x>0<mapsize y=0 TMPMOVE= " << tmpmove)

    p->x=x+1;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p))
    {
        tmpmove=tmp->getTile(x+1,y+1)->getMoves();
        debug("Checked Position: " << x+1 << " " << y+1 )
    }
    p->x=x;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y+1)->getMoves();
        debug("Checked Position: " << x << " " << y+1 )
    }
    p->x=x+1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y)->getMoves();
        debug("Checked Position: " << x+1 << " " << y )
    }
    p->x=x-1;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y+1)->getMoves();
        debug("Checked Position: " << x-1 << " " << y+1 )
    }
    p->x=x-1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x-1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x-1,y)->getMoves();
        debug("Checked Position: " << x-1 << " " << y )
    }
    delete tmppath;
    delete p;
    if (tmpmove!=32767)
        return tmpmove;
    else
        return 0;
  }


  if (x==0 && y>0 && x<mapsize)
  {
    debug("x=0 y>0<mapsize  TMPMOVE= " << tmpmove)

    p->x=x+1;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p))
    {
        tmpmove=tmp->getTile(x+1,y+1)->getMoves();
        debug("Checked Position: " << x+1 << " " << y+1 )
    }
    p->x=x;
    p->y=y+1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y+1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y+1)->getMoves();
        debug("Checked Position: " << x << " " << y+1 )
    }
    p->x=x+1;
    p->y=y;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y)->getMoves();
        debug("Checked Position: " << x+1 << " " << y )
    }

    p->x=x;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x,y-1)->getMoves();
        debug("Checked Position: " << x << " " << y-1 )
    }
    p->x=x+1;
    p->y=y-1;
    if(tmppath->canMoveThere(this,p) && tmp->getTile(x+1,y-1)->getMoves()<tmpmove)
    {
        tmpmove=tmp->getTile(x+1,y-1)->getMoves();
        debug("Checked Position: " << x+1 << " " << y-1 )
    }
    delete tmppath;
    delete p;
    if (tmpmove!=32767)
        return tmpmove;
    else
        return 0;
  }

  delete tmppath;
  delete p;
  return 0;
}

// decrement each armys moves by needed moves to travel

void Stack::decrementMoves(int moves)
{
    debug("decrement_moves()");

    for (iterator it = begin(); it != end(); it++)
    {
        (*it)->decrementMoves(moves);
    }
}

// Purpose: Return the strongest army of a group
// Note: If a hero is present return it. If there are two similar armies
// (two of the same strength, or two heroes) return the first in the sequence.

Army* Stack::getStrongestArmy() const
{
    Army *strongest = 0;
    int highest_strength = 0;

    for (const_iterator it = begin(); it != end(); ++it)
    {
        // if hero
        if (((*it)->getType() == 0) || ((*it)->getType() == 1))
        {
            return *it;
        }
        else if ((*it)->getStrength() > highest_strength)
        {
            highest_strength = (*it)->getStrength();
            strongest = *it;
        }
    }
    return strongest;
}

bool Stack::hasHero() const
{
    Army* a = getStrongestArmy();
    assert(a);
    return ((a->getType() == 0) || (a->getType() == 1));
}

Army* Stack::getFirstHero() const
{
    Army* a = getStrongestArmy();
    if ((a->getType() == 0) || (a->getType() == 1))
    {
        debug("firstHero: " << a->getId())
        return a;
    }
    return NULL;
}

void Stack::getHeroes(culpritsType& dst)
{
    debug("getHeroes - stack = " << this)
    for (const_iterator it = begin(); it != end(); ++it)
    {
        // if hero - add it to the vector
        debug("Army type: " << (*it)->getType())
        if (((*it)->getType() == 0) || ((*it)->getType() == 1))
            dst.push_back((*it)->getId());
    }
}

void Stack::bless()
{
    for (iterator it = begin(); it != end(); it++)
    {
        (*it)->bless();
    }
}

Army* Stack::getFirstUngroupedArmy() const
{
    for (const_iterator it = begin(); it != end(); it++)
    {
        if (!(*it)->isGrouped())
        {
            return *it;
        }
    }
    return 0;
}

bool Stack::isFriend(Player* player) const
{
    return d_player == player;
}

void Stack::nextTurn()
{
    d_defending = false;
    for (iterator it = begin(); it != end(); it++)
    {
        (*it)->resetMoves();
        // TODO: should be moved in a more appropriate place => class Player
        d_player->withdrawGold((*it)->getUpkeep());
        (*it)->heal();
    }
}

bool Stack::save(XML_Helper* helper)
{
    bool retval = true;

    retval &= helper->openTag("stack");
    retval &= helper->saveData("id", d_id);
    if (d_player)
        retval &= helper->saveData("player", d_player->getId());
    else
        retval &= helper->saveData("player", 0);
    retval &= helper->saveData("x", d_pos.x);
    retval &= helper->saveData("y", d_pos.y);
    retval &= helper->saveData("defending", d_defending);

    //save path
    retval &= d_path->save(helper);

    //save armies
    for (iterator it = begin(); it != end(); it++)
        retval &= (*it)->save(helper);

    retval &= helper->closeTag();

    return retval;
}

bool Stack::load(std::string tag, XML_Helper* helper)
{
    if (tag == "path")
    {
        d_path = new Path(helper);

        return true;
    }

    if (tag == "army")
    {
        Army* a = new Army(helper);
        a->setOwner(d_player);
        push_back(a);

        return true;
    }

    if (tag == "hero")
    {
        Hero* h = new Hero(helper);
        h->setOwner(d_player);
        push_back(h);

        return true;
    }

    return false;
}

void Stack::flClear()
{
    for (iterator it = begin(); it != end(); it++)
        delete (*it);
    clear();
}

Stack::iterator Stack::flErase(Stack::iterator object)
{
    delete (*object);
    return erase(object);
}

// End of file
