//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef STRING_TOKENIZER_H
#define STRING_TOKENIZER_H

#include <string>
#include <iterator>
#include <vector>
#include <iostream>

/** The sole use of this class is to analyze a string and append a ".sav" at
  * its end if the string doesn't end like this. Put this in am more generic
  * form and you sort of end up with this class. It is used in the save dialogs.
  */

class stringTokenizer : public std::iterator<std::input_iterator_tag, std::string>
{
    public:
        // CREATORS
        stringTokenizer();
        stringTokenizer(const std::string &_str, const char * _separator =" ");
        stringTokenizer(const stringTokenizer & t);
        ~stringTokenizer();
        // MANIPULATORS
        stringTokenizer & operator++();
        stringTokenizer operator++(int);
        bool operator==(const stringTokenizer & rhs) const;
        bool operator!=(const stringTokenizer & rhs) const;
        // ACCESSORS
        std::string operator*() const;
        std::vector<std::string>* getTokens();
        std::string getFirstToken();
        std::string getLastToken();
        bool isGood();
 
    private:
        // DATA
        const char * d_separator; // Characters separators
        const std::string *d_str;  // Pointer to the string buffer to be tokenized
        std::string::size_type d_start; // Start of the token
        std::string::size_type d_end; //   End of the token

        // PRIVATE ACCESSOR
        void findNext(void);
};

#endif // STRING_TOKENIZER_H

// End of file
