//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "tooltip.h"
#include "defs.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

ToolTip::ToolTip(PG_Widget* parent,const char *text, int screen_x, int screen_y,int max_x,int max_y)
   :PG_ThemeWidget(parent,PG_Rect(screen_x, screen_y,0,0))
{
    debug("Tooltip constructor")
    int pos_x=0;
    int pos_y=0;
    SDL_Color lightyellow = {252, 250, 222};
    SDL_Color black = {0, 0, 0};

    SetSizeByText(4,0,text);
    SetSimpleBackground(true);
    SetBackgroundColor(lightyellow); 
    PG_Label* lab= new PG_Label(this,PG_Rect(2,0,0,0),text);
    lab->SetSizeByText(0,3,text);
    lab->SetFontColor(black);
    lab->SetVisible(true);

    if ((lab->GetTextWidth()+screen_x+4)>max_x)
    {
      pos_x=max_x-lab->GetTextWidth()-10;    
     }
    else pos_x=screen_x;
 
    if ((lab->GetTextHeight()+screen_y+2)>max_y)
    {
      pos_y=max_y-lab->GetTextHeight()-10;    
    }
    else pos_y=screen_y;

    MoveWidget(PG_Rect(pos_x,pos_y,0,0));
    SetSizeByText(4,0,text);
    SetBorderSize(1);

    // Make a black border
    // TODO: a more generic way (this won't become available until a migration
    // to paragui-devel, but we can keep this in mind)
    for (int i = 0; i < 2; i++)
        for (int j = 0; j < 2; j++)
            my_colorBorder[i][j] = black;
    
    // SetCapture, so the tooltip object can detect
    // the moving of the mouse 
    SetCapture();

    //We want the tooltip to be displayed after some time
    debug("Before Addtimer")
    timer=AddTimer(TOOLTIP_WAIT);
    d_movecounter=0;
}

ToolTip::~ToolTip()
{
   debug("Tooltip destructor!");
}

bool ToolTip::eventMouseMotion(const SDL_MouseMotionEvent* event)
{  
    if (d_movecounter>5) 
        delete this;
    else d_movecounter++;
    debug("Value " << d_movecounter);
    return true;
}

bool ToolTip::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
    delete this;
    return true;
}

Uint32 ToolTip::eventTimer (PG_TimerID id, Uint32 interval)
{
    Show();
    debug("Tooltip Show!")
    RemoveTimer(timer);
    return interval;
}
