//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLTIP_H
#define TOOLTIP_H

#include <pgeventobject.h>
#include <pgthemewidget.h>
#include <pgtimerobject.h>
#include <pglabel.h>

/** Simple information tooltip
  * 
  */

class ToolTip : public PG_EventObject, public PG_ThemeWidget,public PG_TimerObject
{ 
    public:
        /** Default constructor
          *
          * The tooltip is displayed with the mouse motion at the  
          * mouse pointer's position
          * 
          * @param text         the text to be displayed
          * @param screen_x     the x_coordinate of the original click
          * @param screen_y     the y coordinate of the original click
          * @param max_x        the max width of the parent widget
          * @param maxy_y       the max height of the parent widget
          */
        ToolTip(PG_Widget* parent,const char* text, int screen_x, int screen_y,int max_x,int max_y);
        ~ToolTip();

    private:
        //! Closes dialog when mouse button is pressed or mouse is moved
        bool eventMouseMotion(const SDL_MouseMotionEvent* event);
        bool ToolTip::eventMouseButtonDown(const SDL_MouseButtonEvent* event);
        //! Shows the tooltip after TOOLTIP_WAIT milliseconds
        Uint32 ToolTip::eventTimer (PG_TimerID id, Uint32 interval);

	PG_TimerID timer;
	int d_movecounter;
};

#endif // TOOLTIP_H

